"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor = __importStar(require("@project-serum/anchor"));
const spl_token_1 = require("@solana/spl-token");
const spl_token_bonding_1 = require("@strata-foundation/spl-token-bonding");
const spl_token_collective_1 = require("@strata-foundation/spl-token-collective");
function retry(func, tries = 0) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield func();
        }
        catch (e) {
            if (tries > 5) {
                throw e;
            }
            console.log("Retrying...");
            yield retry(func, tries + 1);
        }
    });
}
function run() {
    return __awaiter(this, void 0, void 0, function* () {
        console.log(process.env.ANCHOR_PROVIDER_URL);
        anchor.setProvider(anchor.Provider.env());
        const provider = anchor.getProvider();
        const tokenBondingSdk = yield spl_token_bonding_1.SplTokenBonding.init(provider);
        const tokenCollectiveSdk = yield spl_token_collective_1.SplTokenCollective.init(provider);
        const tokenRefs = yield tokenCollectiveSdk.account.tokenRefV0.all();
        const bondingByTokenRef = tokenRefs.filter(tokenRef => tokenRef.account.isClaimed).reduce((acc, tokenRef) => {
            // @ts-ignore
            acc[tokenRef.account.tokenBonding.toBase58()] = Object.assign(Object.assign({}, tokenRef.account), { publicKey: tokenRef.publicKey });
            return acc;
        }, {});
        for (const [tokenBonding, tokenRef] of Object.entries(bondingByTokenRef)) {
            if (!tokenRef.owner) {
                throw new Error("No owner, why?");
            }
            console.log(`Patching bonding ${tokenBonding} to point to ${tokenRef.owner.toBase58()}`);
            const bonding = (yield tokenBondingSdk.getTokenBonding(tokenRef.tokenBonding));
            const baseRoyalties = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, bonding.baseMint, tokenRef.owner);
            const targetRoyalties = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, bonding.targetMint, tokenRef.owner);
            const mintTokenRef = (yield spl_token_collective_1.SplTokenCollective.mintTokenRefKey(tokenRef.mint))[0];
            if (bonding.buyBaseRoyalties.equals(baseRoyalties) || bonding.buyTargetRoyalties.equals(targetRoyalties)) {
                console.log(`Skipping ${tokenBonding} because royalties aleady point to ${tokenRef.owner.toBase58()}`);
                continue;
            }
            const instructions = [];
            if (!(yield tokenBondingSdk.accountExists(baseRoyalties))) {
                instructions.push(spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, bonding.baseMint, baseRoyalties, tokenRef.owner, provider.wallet.publicKey));
            }
            if (!(yield tokenBondingSdk.accountExists(targetRoyalties))) {
                instructions.push(spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, bonding.targetMint, targetRoyalties, tokenRef.owner, provider.wallet.publicKey));
            }
            instructions.push(yield tokenCollectiveSdk.program.instruction.patchRoyalties({
                accounts: {
                    strata: provider.wallet.publicKey,
                    mintTokenRef,
                    tokenBonding: bonding.publicKey,
                    tokenBondingProgram: tokenBondingSdk.programId,
                    baseMint: bonding.baseMint,
                    targetMint: bonding.targetMint,
                    buyBaseRoyalties: baseRoyalties,
                    buyTargetRoyalties: targetRoyalties,
                    sellBaseRoyalties: baseRoyalties,
                    sellTargetRoyalties: targetRoyalties
                }
            }));
            yield retry(() => __awaiter(this, void 0, void 0, function* () {
                yield tokenCollectiveSdk.sendInstructions(instructions, []);
            }));
        }
    });
}
run().catch(e => {
    console.error(e);
    process.exit(1);
});
//# sourceMappingURL=patch.js.map