"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * A script to get holders of a certain mint
 */
const commander_1 = require("commander");
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const program = new commander_1.Command();
const suffleArray = (array) => {
    let currentIndex = array.length, randomIndex;
    // While there remain elements to shuffle.
    while (currentIndex != 0) {
        // Pick a remaining element.
        randomIndex = Math.floor(Math.random() * currentIndex);
        currentIndex--;
        // And swap it with the current element.
        [array[currentIndex], array[randomIndex]] = [
            array[randomIndex],
            array[currentIndex],
        ];
    }
    return array;
};
program
    .version("0.1.0")
    .argument("<mintPublicKey>", "PublicKey of mint")
    .description("select a random wallet from people who hold a sol mint token")
    .action((mintPublicKey, solanaUrl) => __awaiter(void 0, void 0, void 0, function* () {
    let holders = {};
    let entriesBucket = [];
    const mint = new web3_js_1.PublicKey(mintPublicKey);
    const connection = new web3_js_1.Connection((0, web3_js_1.clusterApiUrl)("mainnet-beta"), "confirmed");
    const accounts = yield connection.getParsedProgramAccounts(spl_token_1.TOKEN_PROGRAM_ID, {
        filters: [
            {
                dataSize: 165, // number of bytes
            },
            {
                memcmp: {
                    offset: 0,
                    bytes: mint.toBase58(),
                },
            },
        ],
    });
    console.log(`Found ${accounts.length} token account(s) for mint ${mint.toString()}: `);
    accounts.forEach((a, i) => {
        if (a.account.data.parsed.info.tokenAmount.uiAmount > 0) {
            // @ts-ignore
            if (!holders[a.account.data.parsed.info.owner]) {
                // @ts-ignore
                holders[a.account.data.parsed.info.owner] = {
                    amount: +a.account.data.parsed.info.tokenAmount.uiAmount,
                };
            }
            else {
                // @ts-ignore
                holders[a.account.data.parsed.info.owner].amount +=
                    +a.account.data.parsed.info.tokenAmount.uiAmount;
                // @ts-ignore
            }
        }
    });
    // Create a bucket of owners with their amount of entries
    for (const [key, value] of Object.entries(holders)) {
        entriesBucket.push(...[...Array(value.amount).keys()].map((x) => key));
    }
    console.log(`Shuffling ${entriesBucket.length} entries`);
    // Shuffle the array of entries
    suffleArray(entriesBucket);
    // Select random winner
    console.log(`Winner: ${entriesBucket[Math.floor(Math.random() * (entriesBucket.length - 1))]}`);
}));
program.parse();
//# sourceMappingURL=raffle-winner.js.map